module PLM
  #
  #
  #
  module Model
    #
    # On construit une classe qui derive de ActiveLDAP qui recupere les informations des
    # gestionnaires de laboratoire en read-write
    #
    # le "has_many":admins, creer un "accessor" pour l'objet Branch qui est calcule par une
    # jointure entre la clef "uniqueMember" et l'objet "Plm"
    #
    class Branch < PLM::Model::PlmBackend
      ldap_mapping dn_attribute: 'cn',
      prefix: 'o=admin',
      classes: ['top', 'groupOfUniqueNames']
      has_many :admins,
      class_name: 'PLM::Model::Plm',
      primary_key: 'dn',
      wrap: 'uniqueMember'

      def description
        attribute_present?('description') ? self[:description] : ''
      end

      def to_hash
        people = PLM::Model::Plm.all(filter:"ou=#{o}")
        members = people.select{ |m|
          JSON.parse(m.plmPrimaryAffectation)['ou'].to_i == o.to_i
        }.size
        expires =  people.select{ |m| m.attribute_present?('shadowExpire') }.size
        b = {}
        b[:name]     = description
        b[:id]       = o
        b[:code]     = cn
        b[:count]    = PLM::Model::Plm.count(filter:"ou=#{o}")
        b[:emath]    = {}
        b[:admins]   = []
        admins.each { |a| b[:admins].push({name:a.cn+' '+a.sn,email:a.mail}) }
        b[:count]    = people.count
        b[:expires]  = expires
        b[:members]  = members

        if attribute_present?('ou')
          emath = PLM::Model::Emath.count(filter:"destinationIndicator=#{ou}")
          b[:emath][:id] = ou
          b[:emath][:count] = PLM::Model::Emath.count(filter:"destinationIndicator=#{ou}")
        end

        b
      end

      def short
        {
          code: code,
          description: description,
          id: branchId,
          labId: labId
        } if branchId > 0
      end

      def vpn
        VpnInfo.first(base: dn)
      end

      def code
        cn(true).first
      end

      def branchId
        if attribute_present?('o')
          o.to_i rescue -1
        end
      end

      def labId
        if attribute_present?('ou')
          ou.to_i rescue -1
        end
      end

      class << self
        def entities
          all_entities = []
          all(filter:'o=*').each do |branch|
            all_entities.push({
            name: branch.description,
            id: branch.o(true).first,
            code: branch.cn(true).first
            })
          end
          all_entities.sort_by!{ |entity| entity[:name] } if all_entities.size > 1
          all_entities
        end

        def find_by_id(id=-1)
          if id.is_a?(Integer) && id > 0
            first("o=#{id}")
          end
        end

        def find_by_labId(id=-1)
          if id.is_a?(Integer) && id > 0
            first("ou=#{id}")
          end
        end
      end

    end
  end
end
