module PLM
    module Model
        class Convergence

            attr_accessor :hash, :stringify_keys, :emath_id
            
            def initialize(hash)

                keys = [:spFrom, :IDP, :registrationAuthority, :UserName, :EPPN, :MAIL, :CN, :SN, :givenName, :displayName]

                keys.each do |key|
                    raise 'Missing default keys in convergence hash' unless hash.key?(key)
                end
            
                @emath_id = [171,172,175,176,189]
                
                @hash = hash.slice(*keys)
            
            end

            def compute

                @hash[:MAILI]     = @hash[:MAIL].downcase
                @hash[:MAILS]     = [@hash[:MAILI]]

                @hash[:emathMAILS] = []
                @hash[:title] = []
                @hash[:MSC2010] = []
                @hash[:FROM]   = 'shib'
                @hash[:STATUS] = 'other'
                @hash[:LAB] = []
                user = nil

                if @hash[:MAIL] =~ /.+@.+\..+/i
        
                    # IDP Mathrice
                    if @hash[:IDP].match?(/Mathrice/i) &&
                        @hash[:USER]
                        @hash[:FROM] = 'plm'
                        user = Plm.find(@hash[:USER])
        
                    #IDP Fede
                    else
                        user = Plm.first("mail=#{@hash[:MAIL]}")
                        user = Plm.first("mailAlternateAddress=#{@hash[:MAIL]}") unless user
                    end

                    if user
                        # Si une correspondance d'adresse mél a été trouvée ou que c'est une authentification via l'IdP Mathrice,
                        # le statut passe à ext.
                        @hash[:STATUS] = 'ext'
                        @hash[:USER] = user.uid
                        @hash[:mailLAB] = user.mail

                        #########################################################################
                        #
                        # PLM
                        #
                        #########################################################################
                        @hash[:OU] = []
                        @hash[:OU] = user.ou(true) if user.attribute_present?('ou')
            
                        # difference entre plm et ext
                        @hash[:STATUS] = 'plm' unless @hash[:OU].include?(GUEST)
                        if user.attribute_present?('mailMathrice')
                            @hash[:mailPLM] = user.mailMathrice
                            @hash[:MAILS].push(user.mailMathrice.downcase)
                            @hash[:MAILS].uniq!
                        end
            
                        # Est admin de branche
                        @hash[:ADMIN] = user.is_admin?

                        # Est root
                        @hash[:ROOT]  = user.is_root?
            
                        #######################################################################
                        #
                        # On récupère le(s) labo(s) dans l'annuaire Emath (mail et mailAlternateAddress) à partir de user.mail
                        #
                        #######################################################################
                        title_msc_labs_mails(user.mail)
            
                        #############################################################################
                        #
                        # Intersect PLM with EMATH on PLM mailAlternateAddress
                        # to get labs
                        # On récupère le(s) labo(s) dans l'annuaire Emath (mail et mailAlternateAddress) à partir de user.mailAlternateAddress
                        #
                        #############################################################################
                        if user.attribute_present?('mailAlternateAddress')
                            @hash[:mailAPLM] = user.mailAlternateAddress(true)
                            @hash[:MAILS] = @hash[:MAILS] + @hash[:mailAPLM]
                            @hash[:MAILS].uniq!

                            @hash[:mailAPLM].each do |mail|
                                title_msc_labs_mails(mail)
                            end
                        
                        end
                        
                        if @hash[:LAB].count > 0
                            if @hash[:IDP].match?(/Mathrice/i)
                                @hash[:FROM] = 'plm_emath'
                            else
                                @hash[:FROM] = 'emath'
                            end
                        end
        
                    else
            
                        #########################################################################
                        #
                        # Si aucune correspondance d'adresse mél. n'a été trouvée dans la PLM
                        # on cherche une correspondance dans l'adresse emath
                        #########################################################################
                        title_msc_labs_mails(@hash[:MAIL])

                        @hash[:FROM] = 'emath' if @hash[:LAB].count > 0
                        
                        #Ajouter les id emath
                        if @hash[:STATUS] == 'other' && !@emath_labid.nil?
                            @hash[:LAB].each do |id|
                                @hash[:STATUS] = 'insmi' unless @emath_labid.include?(id)
                            end
                        end
        
                    end

                    @hash[:LAB] = @hash[:LAB].compact.uniq
                end

                if @hash[:STATUS] == 'insmi'
                    @hash[:hasMailInPLMAccounts] = false
                    Emath.all(filter: [ :or, {mail: @hash[:MAIL], mailAlternateAddress: @hash[:MAIL]} ]).each do |e|
                        if e.attribute_present?('mail') &&
                            Emath.all(filter: [ :or, {mail: e.mail, mailAlternateAddress: e.mail} ])
                            @hash[:hasMailInPLMAccounts] = true
                        end
                        if e.attribute_present?('mailAlternateAddress')
                            e.mailAlternateAddress(true).each do |mail|
                                if Plm.first(filter: [ :or, {mail: mail, mailAlternateAddress: mail} ])
                                    @hash[:hasMailInPLMAccounts] = true
                                end
                            end
                        end
                    end
                end

                # Essaye de renseigner l'OU à partir du labid du labo dans la fiche admin du labo de la PLM
                if @hash[:OU].nil? && !@hash[:LAB].nil?
                    @hash[:OU] = []
                    @hash[:LAB].each do |l|
                        if ((adm = Branch.find_by_labId(l.to_i)) && adm.branchId)
                            @hash[:OU].push adm.branchId
                        end
                    end
                    @hash.delete(:OU)if @hash[:OU].count == 0
                end

                @stringify_keys = @hash.collect{|k,v| [k.to_s, v]}.to_h
                @hash
            end
        
            private

            def title_msc_labs_mails(mail)
  
                Emath.all(filter: [ :or, {mail: mail, mailAlternateAddress: mail} ]).each do |entry|
                    @hash[:LAB].push(entry.destinationIndicator)
  
                    msc_regexp = /AMS[0-9]+:(?<code>[0-9]+)/
          
                    @hash[:title] = entry.title if entry.attribute_present?('title')
              
                    entry.descriptions.each do |desc|
                        if desc=~/^\[AMS[0-9]+:/
                            @hash[:MSC2010] = [] unless @hash[:MSC2010]
                            desc.scan(msc_regexp) { |match| @hash[:MSC2010] << match.first.to_i }
                        end
                    end
  
                    if entry.attribute_present?('mail')
                        @hash[:emathMAILS].push(entry.mail.downcase)
                    end
                      
                    if entry.attribute_present?('mailAlternateAddress')
                        @hash[:emathMAILS] += entry.mailAlternateAddress(true).map(&:downcase)
                    end
                    
                end

                @hash[:MSC2010].uniq!
                @hash[:LAB].compact.uniq!
                @hash[:emathMAILS].uniq!

            end
        end
    end
end
