###############################################################################
#
# cf https://activeldap.github.io/activeldap/en/file.tutorial.html
#
# Juggling multiple LDAP connections
#
# In the same vein as the last tip, you can use multiple LDAP connections by
# per class as follows:
#
# irb> anon_class = Class.new(Base)
# => ...
# irb> anon_class.setup_connection
# => ...
# irb> auth_class = Class.new(Base)
# => ...
# irb> auth_class.setup_connection(:password_block => lambda{'mypass'})
# => ...
#
# This can be useful for doing authentication tests and other such tricks.
#
###############################################################################
require 'active_ldap'

module PLM
  #
  #
  #
  module Model
    #
    # Configuration d'un module de log
    #
    # [*WS_NAME*] est une constante qui est recupere par le module PortailsMaths
    #
    # La fonction shared_config surcharge la fonction de shared_config d'origine (qui vient de
    # PLM_Common) pour rajouter la definition de logger
    #
    class ConfigModel
      class << self
        def shared_config(key)
          config          = ::PLM::Model::Config::DIRECTORY[key]
          config
        end
      end
    end
    #
    #
    # RW connections to ldap, please maintain updaterefs on openldap for
    # write permissions if slave connection
    #
    conf = PLM::Model::ConfigModel.shared_config(:ldap_plm_rw) rescue conf {}
    PlmBackend = Class.new(ActiveLdap::Base)
    PlmBackend.setup_connection(conf)
    #
    # Both RO and RW connections to annuaire emath
    # (need to maintain updaterefs on openldap slave)
    #
    conf = PLM::Model::ConfigModel.shared_config(:ldap_emath_rw) rescue conf {}
    EmathBackend = Class.new(ActiveLdap::Base)
    EmathBackend.setup_connection(conf)
    #
    # RW connection to Sup annuaire emath
    #
    confsup = PLM::Model::ConfigModel.shared_config(:ldap_emathsup_rw) rescue confsup {}
    EmathSupBackend = Class.new(ActiveLdap::Base)
    EmathSupBackend.setup_connection(confsup)

    #
    # On construit une classe qui derive de ActiveLDAP qui recupere les informations des
    # Individus dans les laboratoires
    #
  end
end

require_relative 'plm'
require_relative 'branch'
require_relative 'group'
require_relative 'myservices'
require_relative 'emath'
require_relative 'emath_corresp'
require_relative 'vpninfo'
require_relative 'convergence'
