require_relative 'plm_mixin'

module PLM
  #
  #
  #
  module Model
    #
    # Description structure LDAP pour les utilisateurs
    #
    # Le belongs_to est un accessor pour l'objet ActiveLDAP User qui fait l'inverse c'est a
    # dire retourne la liste des branches que manage l'utilisateur
    #

    class Plm < PLM::Model::PlmBackend
      #class AdapterLdap < PLM::Model::PlmBackend

      include PLM::Model::PlmAdapter::InterfaceMixin

      ldap_mapping dn_attribute: 'uid',
      prefix: 'o=People',
      classes: ['posixAccount', 'shadowAccount']
      belongs_to :manage,
      class_name: 'PLM::Model::Branch',
      many: 'uniqueMember',
      primary_key: 'dn'
      belongs_to :branches,
      class_name: 'PLM::Model::Branch',
      many: 'o',
      primary_key: 'ou'
      belongs_to :services,
      class_name: 'PLM::Model::MyService',
      many: 'memberUid',
      primary_key: 'uid'
      belongs_to :emath_manage,
      class_name: 'PLM::Model::EmathCorresp',
      many: 'uniqueMember',
      primary_key: 'dn'

      class << self
        def by_affectation(ou)
          all("ou=#{ou}")
        end
      end

      def adapter
        @plm_ldap = find(uid)
      end

      def add_affectation(attrs)
        aff_id = attrs.fetch(:id,-1).to_i
        if aff_id != GUEST &&
          PLM::Model::Branch.first("o=#{aff_id}") &&
          !affectations.include?(aff_id)
          # Migration vers attributs normalisse a prevoir
          #hash = {id:aff_id, beginsAt:"",endsAt:"",comment:""}
          #hash = {id:aff_id, beginsAt:"",endsAt:"",comment:""}
          hash = build_affectation({}, attrs)
          if attribute_present?('plmSecondaryAffectation')
            self.plmSecondaryAffectation = plmSecondaryAffectation(true).push(hash.to_json)
          else
            self.plmSecondaryAffectation = hash.to_json
          end
          self.ou = ou(true) + [aff_id.to_s]
          save
        end
      end

      def primary_affectation(attrs, isadmin=false)
        if (aff_id = attrs.fetch(:id, -1)) &&
          aff_id != GUEST &&
          PLM::Model::Branch.find_by_id(aff_id)
          #
          # Guest devient membre en affectation primaire
          # ou bien le demandeur est autorise car
          # - admin de la branche principale de la personne
          # - admin de la branche principale et valide la requete
          if (affectation == GUEST || isadmin)

            old_affectation = affectation

            #hash = {id:aff_id, beginsAt:"",endsAt:"",comment:""}
            hash = build_affectation({}, attrs)
            self.plmPrimaryAffectation = hash.to_json

            affs = secondaryAffectations.map { |aff|
              if aff[:id] != aff_id
                hash = build_affectation({}, aff)
                hash.to_json
              end
            }
            if affs.size > 0
              self.plmSecondaryAffectation = affs
            else
              self.plmSecondaryAffectation = nil
            end

            self.ou = (ou(true) - [old_affectation.to_s] + [aff_id.to_s]).uniq
            save
          # Sinon, on positionne le :to
          else

            attrs = primaryAffectation
            attrs[:destinationEntity] = aff_id
            hash = build_affectation({}, attrs)
            self.plmPrimaryAffectation = hash.to_json
            save
          end
        end
      end

      def update_affectation(attrs)
        aff_id = attrs.fetch(:id,-1).to_i
        if affectation == aff_id
          hash = primaryAffectation
          hash = build_affectation(hash, attrs)
          self.plmPrimaryAffectation = hash.to_json
          self.cn   = attrs.fetch(:givenName, cn)
          self.sn   = attrs.fetch(:familyName, sn)
          self.mail = attrs.fetch(:contactEmail, mail)
          save
        elsif affectations.include?(aff_id)
          affs = secondaryAffectations.map do |hash|
            if hash[:id] == aff_id
              hash = build_affectation(hash, attrs)
            else
              hash = build_affectation(hash, {})
            end
            hash.to_json
          end
          self.plmSecondaryAffectation = affs
          save
        end
      end

      def remove_affectation(attrs)
        aff_id = attrs.fetch(:id,-1).to_i
        if aff_id != GUEST
          if affectation == aff_id
            attrs[:id] = GUEST
            hash = build_affectation({}, attrs)
            self.plmPrimaryAffectation = hash.to_json
            self.ou = (ou(true) - [aff_id.to_s] + [GUEST.to_s]).uniq
          elsif affectations.include?(aff_id)
            affs = secondaryAffectations.map do |hash|
              if aff_id != hash[:id]
                hash = build_affectation(hash, {})
                hash.to_json
              end
            end
            self.plmSecondaryAffectation = affs
            self.ou = ou(true) - [aff_id.to_s]
          end
          save
        else
          # raise Error pas possible
        end
      end

      def set_shell(shell)
        self.loginShell=shell
        save
      end

      def update_mail(email)
        if email =~ URI::MailTo::EMAIL_REGEXP
          self.mail=email
          save
        else
          return false
        end
      end

      def vpn
        VpnInfo.first(base: dn)
      end

      #
      # Gestion de l'attribut facultatif
      #
      ### Genere sysem tack too deep !!!
      #def loginShell
      #  return super loginShell
      #  plm_ldap.attribute_present?('loginShell') ? plm_ldap.loginShell : ''
      #end

      def get_plm_labs
        res = []
        ou = JSON.parse(plmPrimaryAffectation)["ou"].to_i
        branches.each do |b|
          name = b.cn.to_s
          desc = b.description
          id = b.o.to_i
          primary = (ou == id)
          labId = b.attribute_present?('ou') ? b.ou.to_i : 0
          res.push({name:name,desc:desc,id:id,labId:labId,primary:primary})
        end
        res
      end

      #
      # Affectation principale
      #
      def affectation
        begin
          primaryAffectation[:id]
        rescue
          '-1'
        end
      end

      def primaryAffectation
        renameKeys(JSON.parse(plmPrimaryAffectation, symbolize_names: true))
      end

      def secondaryAffectations
        affs = []
        if attribute_present?('plmSecondaryAffectation')
          plmSecondaryAffectation(true).collect do |a|
            affs.push(renameKeys(JSON.parse(a, symbolize_names: true)))
          end
        end
        affs
      end

      def workflowTokens(type='')
        res = []
        if attribute_present?('tokenPLM')
          tokenPLM(true).each do |token|
            if type == '' || token.match(/^#{type}/)
              spl = token.split('@@')
              hash = {
                action: spl[0],
                ts: spl[1],
                contactEmail: spl[2],
                givenName: spl[3],
                familyName: spl[4]
              }
              res.push hash
            end
          end
        end
        res
      end

      def add_token(newtoken, one='')
        # On va stocker ce token chez l'utilisateur qui invite
        if attribute_present?('tokenPLM')
          tokens = tokenPLM(true)
          if one != ''
            tokens.delete_if{ |tk| tk.match(/^#{one}/) }
          end
          self.tokenPLM=tokens.push(newtoken)
        else
          add_class("tokenMathrice") unless self.classes.include?("tokenMathrice")
          self.tokenPLM=newtoken
        end
        save
      end


      ###############################################################################
      ## Fonctions de suppression d'un token d'invitation
      ###############################################################################
      def remove_token(token)
        # On supprime le token qui match l'expression
        if attribute_present?('tokenPLM')
          self.tokenPLM=tokenPLM(true).delete_if{ |tk| tk.match(/^#{token}/) }
          save
        end
      end

      def change_password(password)
        self.userPassword = password
        self.add_class('shadowAccount') unless classes.include?('shadowAccount')
        self.shadowExpire='' if attribute_present?('shadowExpire')
        self.shadowMax=190 unless attribute_present?('shadowMax')
        self.shadowMin=0 unless attribute_present?('shadowMin')
        self.shadowWarning=180 unless attribute_present?('shadowWarning')
        self.shadowLastChange=Time.now.to_i/86400
        save
      end

      def expired?
        attribute_present?('shadowExpire')
      end

      def is_admin?
        manage.map { |isadmin| isadmin.o ? isadmin.o : nil }.compact
      end

      def is_root?
        !PLM::Model::Branch.first(
          filter:"(&(cn=ldap)(uniqueMember=#{self.dn}))"
        ).nil?
      end

      def is_guest?
        affectation == GUEST
      end

      def is_primary?(id)
        id = id.to_i
        affectation == id
      end

      def is_secondary?(id)
        id = id.to_i
        affectations.include?(id) && !is_primary?(id)
      end

      #
      # Gestion de l'attribut multivalué
      #
      def affectations
        ou(true).map{ |a| a.to_i}
      end

      def mailAlternateAddresses
        attribute_present?('mailAlternateAddress') ? mailAlternateAddress(true) : []
      end

      def add_or_create_mailAlternateAddress(email)
        emails = mailAlternateAddresses.push(email) unless mailAlternateAddresses.include?(email)
        update_mailAlternateAddress(emails)
      end

      def update_mailAlternateAddress(emails)
        add_class("qmailUser") unless classes.include?("qmailUser")
        self.mailAlternateAddress = emails
        save
      end

      def add_or_create_sshPublicKey(ldapkeys)
        add_class("ldapPublicKey") unless classes.include?("ldapPublicKey")
        self.sshPublicKey = ldapkeys
        save
      end
      #
      # Gestion de l'attribut multivalué
      #
      def sshPublicKeys
        attribute_present?('sshPublicKey') ? sshPublicKey(true) : []
      end

      #
      # Ajout d'une clé SSH
      #
      def add_sshKey(req)
        sshKeys(req,'add')
      end
      #
      # Renommage d'une clé SSH
      #
      def set_sshKey(req)
        sshKeys(req,'mod')
      end
      #
      # Suppression d'une clé SSH
      #
      def del_sshKey(req)
        sshKeys(req,'del')
      end

      def all_but_uid_sshPublicKeys(uid)
        all(filter:"(&(!(uid=#{uid}))(sshPublicKey=*))")
      end

      private

      def renameKeys(hash)
        hash[:beginsAt] = hash.delete :'date-debut' if hash.has_key? :'date-debut'
        hash[:endsAt] = hash.delete :'date-fin' if hash.has_key? :'date-fin'
        hash[:comment] = hash.delete :commentaire if hash.has_key? :commentaire
        hash[:id] = hash.delete :ou if hash.has_key? :ou
        hash[:destinationEntity] = hash.delete :to if hash.has_key? :to
        hash[:destinationEntity] = hash[:destinationEntity].to_i if hash.has_key? :destinationEntity
        hash[:id] = hash[:id].to_i
        hash
      end

      # def build_affectation(hash, attrs)
      #   hash = renameKeys(hash)
      #   hash[:beginsAt] = attrs.fetch(:beginsAt,hash[:beginsAt])
      #   hash[:endsAt] = attrs.fetch(:endsAt,hash[:endsAt])
      #   hash[:comment] = attrs.fetch(:comment,hash[:comment])
      #   hash
      # end
      def build_affectation(hash, attrs)
        if attrs.has_key? :destinationEntity
          hash["to"] = attrs.fetch(:destinationEntity,hash[:destinationEntity])
        end
        hash["ou"] = attrs.fetch(:id,hash[:id])
        hash["date-debut"] = attrs.fetch(:beginsAt,hash[:beginsAt])
        hash["date-fin"] = attrs.fetch(:endsAt,hash[:endsAt])
        hash["commentaire"] = attrs.fetch(:comment,hash[:comment])
        hash = hash.select do |k, _v|
          %w[
            to
            ou
            date-debut
            date-fin
            commentaire
          ].include?(k)
        end
        hash
      end
    end
  end
end
