require 'resolv'
require 'ipaddr'
require "base64"
module PLM
  #
  #
  #
  module Model

    class VpnInfo < PLM::Model::PlmBackend

      ldap_mapping dn_attribute: 'cn',
      #prefix: 'o=people',
      classes: ['plmVPN']

      class << self

        def parse_route(route)
          re = /([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)(\/[^\#]+)?(\#(.*))?/
          route.scan(re) do |ip,mask,comm,desc|
            begin
              return "#{ip}#{mask}",Resolv.getname(ip),desc
            rescue Resolv::ResolvError => e
              return "#{ip}#{mask}",'',desc
            end
          end
        end

        def get_real_dn(entry)
          return entry.dn.to_s.gsub(
          ',ou=VpnInfo,dc=mathrice,dc=fr',''
          ).strip
        end

        def get_routes(entity,user=nil)
          routes     = []
          global_ips = []
          entity_ips = []
          rdn = "uid=#{user}," if user
          global_config = first(attribute: 'plmvpnRoute',
                                base:'cn=ldap,o=admin,dc=mathrice,dc=fr')
          entity_config = first(filter:"ou=#{entity}",
                                attribute: 'plmvpnRoute',
                                base:'o=admin,dc=mathrice,dc=fr')
          users_config = all(filter:"ou=#{entity}",
                               attribute: 'plmvpnRoute',
                               base:"#{rdn}o=people,dc=mathrice,dc=fr")

          global_dn = get_real_dn(global_config)

          global_config.plmvpnRoute(true).each do |r|
            ip, fqdn, desc  = parse_route(r)
            global_ips.push IPAddr.new(ip)
            routes.push({
              id: Base64.strict_encode64("#{global_dn}##{r}").strip,
              route:ip,
              fqdn:fqdn,
              desc: desc,
              type:{cat:'global',id:'global'},
              status:'unique'
              })
            routes.sort!{ |r1,r2| IPAddr.new( r1[:route] ) <=> IPAddr.new( r2[:route])}
          end

          routes_temp = []
          entity_dn = get_real_dn(entity_config)

          entity_config.plmvpnRoute(true).each do |r|
            ip_temp, fqdn, desc  = parse_route(r)
            ip_addr = IPAddr.new(ip_temp)
            entity_ips.push ip_addr
            status = global_ips.select { |ip| ip.include?(ip_addr) }.size == 0
            routes_temp.push({
              id: Base64.strict_encode64("#{entity_dn}##{r}").strip,
              route:ip_temp,
              fqdn:fqdn,
              desc: desc,
              type:{cat:'entity',id:entity},
              status:status ? 'unique':'global'
              })
            routes_temp.sort!{ |r1,r2| IPAddr.new( r1[:route] ) <=> IPAddr.new( r2[:route])}
            routes = routes + routes_temp
          end

          routes_temp = []
          users_config.each do |u|
            if u.attribute_present?('plmvpnRoute')
              u_dn = get_real_dn(u)
              u.plmvpnRoute(true).each do |r|
                ip_temp, fqdn, desc  = parse_route(r)
                ip_addr = IPAddr.new(ip_temp)
                status = if global_ips.select{ |ip| ip.include?(ip_addr) }.size == 0
                  if entity_ips.select{ |ip| ip.include?(ip_addr) }.size == 0
                    'unique'
                  else
                    'entity'
                  end
                else
                  'global'
                end
                uid = u.dn.parent.rdns[0]['uid']
                routes.push({
                  id: Base64.strict_encode64("#{u_dn}##{r}").strip,
                  route:ip_temp,
                  fqdn:fqdn,
                  desc: desc,
                  type:{cat:'user',id:uid},
                  status:status
                  })
              end
            end
          end
          routes_temp.sort!{ |r1,r2| IPAddr.new( r1[:route] ) <=> IPAddr.new( r2[:route])}
          routes + routes_temp
        end
      end

      def package
        self.plmvpnZip
      end

      def routes
        h = []
        re = /([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)(\#(.*))?/
        self.plmvpnRoute(true).each do |route|
          route.scan(re) do |ip,comm,desc|
            begin
              fqdn = Resolv.getname(ip)
            rescue Resolv::ResolvError => e
              fqdn = "unresolved IP"
              desc = desc ? desc : "PLEASE REMOVE THIS IP"
            end
            h.push({
              ip:ip,
              description:desc ? desc : '',
              fqdn:fqdn
              })
            end
          end
          h
        end

        def add_route(ip, desc)
          edit_route(ip, desc, false) unless already_set?(ip)
        end

        def edit_route(ip, desc)
          edit_route(ip, desc, false) if in_conf?(ip)
        end

        def remove_route(ip, desc)
          edit_route(ip, desc, true) if in_conf?(ip)
        end

        #
        # Test if ip is already set
        # if cn=nil : hierachical search from myself to global conf
        # if cn is set : this branch and global conf (not myself)
        #
        def already_set?(ip, cn=nil)
          if cn.nil?

            in_ldap = in_conf?(ip, 'cn=ldap') || in_conf?(ip)
            if self.dn.parent.rdns[0] == 'o=people'
              filter = "o=#{self.ou}"
              in_ldap || in_conf?(ip, "o=#{self.ou}")
            else
              in_ldap
            end
          else
            in_conf?(ip, 'cn=ldap') || in_conf?(ip, "cn=#{cn}")
          end
        end

        private

        def edit_route(ip, desc, remove=false)
          # Normalize/verify IP Address syntax
          ip = IPAddr.new(ip).to_s

          route = encode_route(ip, desc)
          new_routes = remove ? [] : route

          plmvpnRoute(true).each do |r|
            new_routes.push(r) unless r.match(/^#{ip}(?![0-9])/)
          end

          self.plmvpnRoute = new_routes
          save
        end

        def encode_route(ip, desc)
          ip = "#{ip}##{desc}" if desc.match(/\w+/)
          [ip]
        end

        def in_conf?(ip, filter=nil)
          # Normalize/verify IP Address syntax
          ip = IPAddr.new(ip).to_s

          entity = if filter.nil?
            self
          else
            PLM::Model::Branch.first(filter).vpn
          end
          entity.plmvpnRoute(true).select { |r|
            r.match(/^#{ip}(?![0-9])/)
          }.count == 1
        end

      end
    end
  end
